
strength_vals = [50, 100, 200, 300, 400, 500, 600, 700, 800, 900, 950]
strength_cnt = len(strength_vals)

# format is: ("color_name", switch-to-white-text-strength)
colors = [
	# standard colors
	("red", 500),
	("green", 500),
	("blue", 500),
	("cyan", 500),
	("pink", 500), # == magenta
	("yellow", 1e9), # notice that this never uses white text
	("orange", 800),
	# slightly exotic
	("purple", 300),
	("teal", 500),
	("slate", 400),
	("amber", 1e9),
	("indigo", 300),
	# skeleton-theme aliasses; thresholds accord to "usual" choices
	("primary", 500),
	("secondary", 500),
	("tertiary", 500),
	("success", 500),
	("warning", 1e9),
	("error", 500),
	("surface", 400), 
	# these are sepperated because they dont support [strength/factor] syntax
	("black", 500),
	("white", 500)
]

def get_inverse_idx(strength_idx):
	inv_idx = (strength_cnt - 1 - strength_idx)
	return inv_idx
def get_contrast_idx(strength_idx, contrast_level, flip_threshold):
	# if we are above the threshold, we need a brighter contrast
	strength = strength_vals[strength_idx]
	if (strength >= flip_threshold):
		contrast_idx = max(strength_idx - contrast_level, 0)
	if (strength < flip_threshold):
		contrast_idx = min(strength_idx + contrast_level, strength_cnt - 1)
	return contrast_idx

def get_strength(strength_idx):
	return strength_vals[strength_idx]
def get_inverse(strength_idx):
	return strength_vals[get_inverse_idx(strength_idx)]
def get_contrast(strength_idx, contrast_level):
	return strength_vals[get_contrast_idx(strength_idx, contrast_level, 500)]

def get_text_color(color, strength_idx):
	# black is a special case, where strength is reversed...
	strength = get_strength(strength_idx)
	if (color[0] == "black"):
		strength = get_inverse(strength_idx)
	if (strength >= color[1]):
		return "white"
	return "black"
def get_text_color_toned(color, strength_idx):
	toned_strength = strength_vals[get_contrast_idx(strength_idx, 5, color[1])]
	return f"{color[0]}-{toned_strength}"

# and for contrast / text-color / text-color-toned, also some inverse funcs
def get_inverse_contrast(strength_idx, contrast_level):
	inv_idx = get_inverse_idx(strength_idx)
	return get_contrast(inv_idx, 2)
def get_inverse_text_color(color, strength_idx):
	inv_idx = get_inverse_idx(strength_idx)
	return get_text_color(color, inv_idx)
def get_inverse_text_color_toned(color, strength_idx):
	inv_idx = get_inverse_idx(strength_idx)
	return get_text_color_toned(color, inv_idx)

with open("extended.postcss", "w") as file:
	file.write("\n@layer components {\n")

	# first, create the missing interpolation values between white and black:
	file.write("\n")
	file.write(f"\t/* color interpolations for black + white */\n")
	# black & white will both be remapped to the "neutral" color. white-50 will become neutral-50 == white,
	# black-50 will become neutral-950 == black --> black is SWITCHED AROUND ! 50 is the darkest variant !!!
	file.write("\n")
	for strength_idx in range(strength_cnt):
		strength = get_strength(strength_idx)
		inverse = get_inverse(strength_idx)
		# for opacity in ["", "/20", "/40"]:
		for opacity in [""]:

			variant_str  = f"\t.bg-black-{inverse}{opacity} {{ @apply "
			variant_str += f"bg-neutral-{inverse}{opacity} "
			variant_str += f"; }}\n"
			file.write(variant_str)
			variant_str  = f"\t.bg-white-{strength}{opacity} {{ @apply "
			variant_str += f"bg-neutral-{strength}{opacity} "
			variant_str += f"; }}\n"
			file.write(variant_str)

	for color in colors:
		file.write("\n")
		file.write(f"\t/* {color[0]} */\n")

		for strength_idx in range(0, strength_cnt):
			file.write("\n")
			strength = get_strength(strength_idx)
			contrast = get_contrast(strength_idx, 2)
			text_color = get_text_color(color, strength_idx)
			toned_text_color = get_text_color_toned(color, strength_idx)

			inverse = get_inverse(strength_idx)
			inv_contrast = get_inverse_contrast(strength_idx, 2)
			inv_text_color = get_inverse_text_color(color, strength_idx)
			inv_toned_text_color = get_inverse_text_color_toned(color, strength_idx)
			
			# filled variants (flat color)
			variant_str  = f"\t.variant-filled-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength} "
			variant_str += f"text-{text_color} "
			variant_str += f"dark:bg-{color[0]}-{inverse} "
			variant_str += f"dark:text-{inv_text_color} "
			variant_str += f"; }}\n"
			file.write(variant_str)

			# soft variants (flat color + toned-text)
			variant_str  = f"\t.variant-soft-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength} "
			variant_str += f"text-{toned_text_color} "
			variant_str += f"dark:bg-{color[0]}-{inverse} "
			variant_str += f"dark:text-{inv_toned_text_color} "
			variant_str += f"; }}\n"
			file.write(variant_str)


			# outline-only variants (fully transparent + outline)
			variant_str  = f"\t.variant-outline-only-{color[0]}-{strength} {{ @apply "
			# variant_str += f"bg-transparent "
			variant_str += f"ring-[2px] "
			variant_str += f"ring-{color[0]}-{strength} "
			# variant_str += f"dark:bg-transparent "
			variant_str += f"dark:ring-[2px] "
			variant_str += f"dark:ring-{color[0]}-{inverse} "
			variant_str += f"; }}\n"
			file.write(variant_str)

			# outline-soft variants (soft color fill + outline; outline is slightly contrasty)
			variant_str  = f"\t.variant-outline-soft-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength} "
			variant_str += f"text-{toned_text_color} "
			variant_str += f"ring-[2px] "
			variant_str += f"ring-{color[0]}-{contrast} "
			variant_str += f"dark:bg-{color[0]}-{inverse} "
			variant_str += f"dark:text-{inv_toned_text_color} "
			variant_str += f"dark:ring-[2px] "
			variant_str += f"dark:ring-{color[0]}-{inv_contrast} "
			variant_str += f"; }}\n"
			file.write(variant_str)

			# outline-filled variants (flat color fill + outline; outline is slightly contrasty)
			variant_str  = f"\t.variant-outline-filled-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength} "
			variant_str += f"text-{text_color} "
			variant_str += f"ring-[2px] "
			variant_str += f"ring-{color[0]}-{contrast} "
			variant_str += f"dark:bg-{color[0]}-{inverse} "
			variant_str += f"dark:text-{inv_text_color} "
			variant_str += f"dark:ring-[2px] "
			variant_str += f"dark:ring-{color[0]}-{inv_contrast} "
			variant_str += f"; }}\n"
			file.write(variant_str)

			# ringed variants (fully-transparent + slim outline)
			variant_str  = f"\t.variant-ringed-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-transparent "
			variant_str += f"ring-[1px] "
			variant_str += f"ring-{color[0]}-{strength} "
			variant_str += f"dark:bg-transparent "
			variant_str += f"dark:ring-[1px] "
			variant_str += f"dark:ring-{color[0]}-{inverse} "
			variant_str += f"; }}\n"
			file.write(variant_str)


			# if the black + white background sub-classes are making problems, this blocks them from being used
			if (color[0] == "white" or color[0] == "black"):
				continue

			# ghost variants (tinted seethrough + thick outline)
			variant_str  = f"\t.variant-ghost-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength}/20 "
			variant_str += f"ring-[2px] "
			variant_str += f"ring-{color[0]}-{strength} "
			variant_str += f"dark:bg-{color[0]}-{inverse}/20 "
			variant_str += f"dark:ring-[2px] "
			variant_str += f"dark:ring-{color[0]}-{inverse} "
			variant_str += f"; }}\n"
			file.write(variant_str)

			# glass variants (blurry seethrough + toned-text)
			variant_str  = f"\t.variant-glass-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength}/40 "
			variant_str += f"text-{toned_text_color} "
			variant_str += f"dark:bg-{color[0]}-{inverse}/40 "
			variant_str += f"dark:text-{inv_toned_text_color} "
			variant_str += f"backdrop-blur-lg "
			variant_str += f"; }}\n"
			file.write(variant_str)

			# seethrough variants (clear seethrough + text)
			variant_str  = f"\t.variant-seethrough-{color[0]}-{strength} {{ @apply "
			variant_str += f"bg-{color[0]}-{strength}/40 "
			variant_str += f"text-{get_text_color(color, strength_idx)} "
			variant_str += f"dark:bg-{color[0]}-{inverse}/40 "
			variant_str += f"dark:text-{inv_text_color} "
			variant_str += f"; }}\n"
			file.write(variant_str)
		
	file.write("\n}\n\n")


