{
  description = "Grady devenv";

  inputs = {
    devenv-root = {
      url = "file+file:///dev/null";
      flake = false;
    };
    flake-parts.url = "github:hercules-ci/flake-parts";
    # nixpkgs.url = "github:cachix/devenv-nixpkgs/rolling";
    nixpkgs.url = "github:NixOS/nixpkgs/nixos-unstable";
    devenv.url = "github:cachix/devenv";
    nix2container.url = "github:nlewo/nix2container";
    nix2container.inputs.nixpkgs.follows = "nixpkgs";
    mk-shell-bin.url = "github:rrbutani/nix-mk-shell-bin";
  };

  nixConfig = {
    extra-trusted-public-keys = "devenv.cachix.org-1:w1cLUi8dv3hnoSPGAuibQv+f9TZLr6cv/Hm9XgU50cw=";
    extra-substituters = "https://devenv.cachix.org";
  };

  outputs = inputs@{ flake-parts, devenv-root, ... }:
    flake-parts.lib.mkFlake { inherit inputs; } {
      imports = [
        inputs.devenv.flakeModule
      ];
      systems = [ "x86_64-linux" "i686-linux" "x86_64-darwin" "aarch64-linux" "aarch64-darwin" ];

      perSystem = { config, self', inputs', pkgs, system, ... }: {
        # Per-system attributes can be defined here. The self' and inputs'
        # module parameters provide easy access to attributes of the same
        # system.

        # Equivalent to  inputs'.nixpkgs.legacyPackages.hello;
        packages.default = pkgs.hello;

        devenv.shells.default =
          let
            surrealVersionTag = "surrealdb/surrealdb:v2.0.0-alpha.10";
          in
          {

            devenv.root =
              let
                devenvRootFileContent = builtins.readFile devenv-root.outPath;
              in
              pkgs.lib.mkIf (devenvRootFileContent != "") devenvRootFileContent;

            name = "Grady";

            imports = [
              # This is just like the imports in devenv.nix.
              # See https://devenv.sh/guides/using-with-flake-parts/#import-a-devenv-module
              # ./devenv-foo.nix
            ];

            # https://devenv.sh/reference/options/
            packages = [ config.packages.default ] ++ (with pkgs; [
              nodePackages.typescript-language-server
              svelte-language-server
              tailwindcss-language-server
            ]);

            scripts = {
              # dev.exec = "bun run dev &";
              db_dev.exec = "docker run -d --rm --pull always --name surrealdb -p 8000:8000 ${surrealVersionTag} start -u root -p root";
              db_persist.exec = ''
                mkdir -p dev_data
                chmod 777 dev_data
                docker run -d -u root --rm --pull always --name surrealdb -p 8000:8000 -v ./dev_data:/dev_data ${surrealVersionTag} start file:/dev_data/dev.db -u root -p root
                # surreal start -u root -p root file:./dev_data/dev.db
              '';
            };

            pre-commit.hooks = {
              eslint.enable = true;
              prettier.enable = true;
              prettier.settings = {
                configPath = ".prettierrc";
              };
            };

            languages = {
              javascript = {
                enable = true;
                bun.enable = true;
                npm.enable = true;
              };
              typescript.enable = true;
            };
          };
      };
      flake = {
        # The usual flake attributes can be defined here, including system-
        # agnostic ones like nixosModule and system-enumerating ones, although
        # those are more easily expressed in perSystem.
      };
    };
}
