// one instance of feedback on a singular of code
export class FeedbackEntry {
	content: string = '';
	author: string = '';
	timestamp: Date = new Date();

	constructor(content: string, author: string) {
		this.author = author;
		this.update_content(content);
	}

	update_content(content: string) {
		this.content = content;
		this.timestamp = new Date();
	}
}

// Feedback for one singular line of code from a submission; may contain many opinions
export class FeedbackLine {
	line_num: number = 0;
	feedback_entries: FeedbackEntry[] = [];

	constructor(line_num: number) {
		this.line_num = line_num;
	}

	add(entry: FeedbackEntry) {
		this.feedback_entries.push(entry);
		this.sort_ascending();
	}
	sort_ascending() {
		this.feedback_entries.sort((a, b) => {
			if (a.timestamp < b.timestamp) return -1;
			if (a.timestamp > b.timestamp) return +1;
			return 0;
		});
	}

	get_entry(author: string) {
		let feedback_entry = this.feedback_entries.find((entry) => entry.author === author);
		// may be undefined
		return feedback_entry;
	}
}

// one entire Feedback-Object, covering of one entire Submission
export class Feedback {
	feedback_lines: FeedbackLine[] = [];

	// this func will also add in new instances if they dont exist
	update_line(line_num: number, content: string, author: string) {
		author = String(content[0]);
		// if the line does not exist, create it
		let target_line = this.get_line(line_num);
		if (target_line === undefined) {
			target_line = new FeedbackLine(line_num);
			this.feedback_lines.push(target_line);
		}
		// if the line does not contain an entry
		// corresponding to the requested author, create it
		let target_entry = target_line.get_entry(author);
		if (target_entry === undefined) {
			target_line.add(new FeedbackEntry(content, author));
			return;
		}
		// otherwise, update the existing line
		target_entry.update_content(content);
		target_line.sort_ascending();
	}

	get_line(line_num: number) {
		let feedback_line = this.feedback_lines.find((entry) => entry.line_num === line_num);
		// may be undefined
		return feedback_line;
	}
}
