import { getDb } from '$lib/surreal.svelte';
import type { User } from '$lib/surreal/schema';
import type { RecordId } from 'surrealdb';
import type { PageLoad } from './$types';
import type Surreal from 'surrealdb';

export const getSubmissionsOverview = async (assignment: RecordId) => {
	const db = await getDb();
};

// fetching all students from DB
export const getAllStudents = async () => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });
	
	const [student_array] = await db.query(`
		SELECT
			record::id(id) AS id,
			alias,
			(
				SELECT
					record::id(id) AS id,
					code,
					stage,
					tests,
					(
						SELECT
							record::id(id) AS id,
							name,
							full_score
						FROM ->assignment
					) AS assignment
				FROM ->submission
				ORDER BY assignment
			) AS submissions
		FROM student;
	`);
	// "ORDER BY assignment" statement for submission-selection causes surreal to panic,
	// but would be good for consistency 
	//
	// alternatively: PARALLEL and ORDER BY clash somehow
	if (student_array === undefined) {
		console.log(`no students available on DB !`)
		return undefined
	}
	return student_array
}

// fetching student that belongs to ID from DB
export const getStudentByID = async (student_id: string) => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });
	
	const [student_array] = await db.query(`
		SELECT
			record::id(id) AS id,
			alias,
			(
				SELECT
					record::id(id) AS id,
					code,
					stage,
					tests,
					(
						SELECT
							record::id(id) AS id,
							name,
							full_score
						FROM ->assignment
					) AS assignment,
					(
						SELECT
							record::id(id) AS id,
							alias
						FROM <-student
					) as submitter
				FROM ->submission
				ORDER BY assignment
			) AS submissions
		FROM student
		WHERE record::id(id) == "${student_id}";
	`);
	// "ORDER BY assignment" statement for submission-selection causes surreal to panic,
	// but would be good for consistency 
	if (student_array === undefined) {
		console.log(`no student available for ID ${student_id} !`)
		return undefined
	}
	return student_array[0]
}

// fetching all submissions that belong to one assignment from DB
export const getSubmissionsByAssignment = async (assignment_id: string) => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });
	
	const [submission_array] = await db.query(`
		SELECT
			record::id(id) AS id,
			code,
			stage,
			tests,
			(
				SELECT
					record::id(id) AS id,
					name,
					full_score
				FROM ->assignment
			) AS assignment,
			(
				SELECT
					record::id(id) AS id,
					alias
				FROM <-student
			) as submitter
		FROM submission
		PARALLEL;
	`);
	if (submission_array === undefined) {
		console.log(`no submissions available for assignment ${assignment_id} !`)
		return undefined
	}
	const matching_submissions = submission_array.filter((submission) =>
		submission.assignment[0].id === assignment_id
	)
	// console.log(matching_submissions)
	return matching_submissions
}

// fetching all submissions that belong to one student from DB
export const getSubmissionsByStudent = async (student_id: string) => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });
	
	const [submission_array] = await db.query(`
		SELECT
			record::id(id) AS id,
			code,
			stage,
			tests,
			(
				SELECT
					record::id(id) AS id,
					name,
					full_score
				FROM ->assignment
			) AS assignment,
			(
				SELECT
					record::id(id) AS id,
					alias
				FROM <-student
			) as submitter
		FROM submission
		PARALLEL;
	`);
	if (submission_array === undefined) {
		console.log(`no submissions available for student ${student_id} !`)
		return undefined
	}
	const matching_submissions = submission_array.filter((submission) =>
		submission.submitter[0].id === student_id
	)
	// console.log(submission_array)
	return matching_submissions
}

// fetching all a submission by its ID from DB
export const getSubmissionsByID = async (submission_id: string) => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });
	
	const [submission_data] = await db.query(`
		SELECT
			record::id(id) AS id,
			code,
			stage,
			tests,
			(
				SELECT
					record::id(id) AS id,
					name,
					full_score
				FROM ->assignment
			) AS assignment,
			(
				SELECT
					record::id(id) AS id,
					alias
				FROM <-student
			) as submitter
		FROM submission
		WHERE record::id(id) = "${submission_id}"
		PARALLEL;
	`);
	if (submission_data === undefined) {
		console.log(`no submission matches the ID ${submission_id} !`)
		return undefined
	}
	return submission_data[0]
}

export const getAssignmentByID = async (assignment_id: string) => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });

	const [assignment_data] = await db.query(`
		SELECT
			record::id(id) AS id,
			name,
			description,
			programming_language,
			full_score,
			solution,
			(
				SELECT
					stage,
					count() AS total
				FROM <-submission
				GROUP BY stage
			) AS submission_state
		FROM assignment
		WHERE record::id(id) = "${assignment_id}"
		PARALLEL;
	`);
	if (assignment_data === undefined) {
		console.log(`no assignment matches the ID ${assignment_id} !`)
		return undefined
	}
	return assignment_data[0]
}

// fetching all assignments from DB
export const getAllAssignments = async () => {
	const db = await getDb();
	// TEMP
	await db.signin({ username: 'root', password: 'root' });

	const [assignment_type_array] = await db.query(`
		SELECT
			record::id(id) AS id,
			name,
			full_score,
			(SELECT
				stage,
				count() AS total
					FROM <-submission
					GROUP BY stage
			) AS submission_state
		FROM assignment
		PARALLEL;
	`);
	if (assignment_type_array === undefined) {
		console.log(`no assignments available on DB !`)
		return undefined
	}
	return assignment_type_array
}

// export const executeQuery = async <T>(
//   queryFn: (db: Surreal) => Promise<T>,
//   config: DbConfig = DEFAULT_CONFIG
// ): Promise<T> => {
//   try {
//     const db = await getDb(config);
//     return await queryFn(db);
//   } catch (error) {
//     console.error('Error executing query:', error);
//     throw error;
//   }
// };

// export const getUserById = async (userId: string): Promise<User | null> => {
//   return executeQuery(async (db) => {
//     const [result] = await db.query<User[]>('SELECT * FROM user WHERE id = $id', { id: userId });
//     return result.length > 0 ? result[0] : null;
//   });
// };

// export const updateUser = async (userId: string, data: Partial<User>): Promise<void> => {
//   return executeQuery(async (db) => {
//     await db.update(`user:${userId}`).content(data);
//   });
// };

// export const selectUsers = async (): Promise<User[]> => {
//   return executeQuery(async (db) => {
//     return await db.select<User[]>('user');
//   });
// };

// import { db } from "$lib/server/db";
// import { Gap, PreparedQuery, type RecordId, type StringRecordId } from "surrealdb"
// import type Surreal from "surrealdb"

// const module = new Gap<StringRecordId | RecordId>();
// const assignmentsQuery = new PreparedQuery(
//   `SELECT * FROM assignment WHERE module == $module`,
//   { module }
// )
// export const getAssignments = async (moduleId: StringRecordId | RecordId) =>
//   await db.getConnection().query(assignmentsQuery, [module.fill(moduleId)])

// const assignment = new Gap<StringRecordId | RecordId>();
// const submissionsQuery = new PreparedQuery(
//   `SELECT <-? AS submissions FROM ONLY $assignment`,
//   { assignment }
// )
// export const getSubmissions = async (assignmentId: StringRecordId | RecordId) => {
//   console.log(`ass id: ${assignmentId}`)
//   return await db.getConnection().query(submissionsQuery, [assignment.fill(assignmentId)])
// }
