import { browser } from '$app/environment';
import pino, { type LoggerOptions, type Logger } from 'pino';
import { get, readable } from 'svelte/store';

const loggerInstance = (() => {
	let options: LoggerOptions = browser
		? {
				browser: { asObject: false },
				level: 'debug',
				formatters: {
					level: (label) => {
						return { level: label.toUpperCase() };
					}
				},
				transport: {
					target: 'pino-pretty',
					options: {
						colorize: true,
						levelFirst: true,
						translateTime: true
					}
				}
			}
		: {
				level: 'debug',
				formatters: {
					level: (label) => {
						return { level: label.toUpperCase() };
					}
				},
				transport: {
					target: 'pino-pretty',
					options: {
						colorize: true,
						levelFirst: true,
						translateTime: true
					}
				}
			};

	return pino(options);
})();

// switch (NODE_ENV) // TODO set log level according to server environment (dev/staging/production)

const pinoLogger = readable<Logger>(loggerInstance);

export const logger = get(pinoLogger);
