import { z } from 'zod';
import { getDb } from './surreal.svelte';
import { StringRecordId } from 'surrealdb';

export const USER_TABLE = 'user';

export enum AccessKind {
	Tutor = 'tutor',
	Reviewer = 'reviewer'
}

export const userSchema = z.object({
	id: z.string(),
	email: z.string().email(),
	password: z.string(),
	access: z.nativeEnum(AccessKind)
});

export const createUserSchema = userSchema.extend({
	id: userSchema.shape.id.optional()
});

export type User = z.infer<typeof userSchema>;
export type CreateUser = z.infer<typeof createUserSchema>;

export const getByEmail = async (email: string) => {
	const db = await getDb();
	if (db == undefined) {
		throw new Error('Db not initialized');
	}
	// TODO select by email using an index
	const users = await db.select<User>(USER_TABLE);
	const user = users.find((u) => u.email == email);
	return user;
};

export const getById = async (id: string) => {
	//: Promise<ActionResult<User>> => {
	const db = await getDb();
	if (db == undefined) {
		throw new Error('Db not initialized');
	}
	const recordId = new StringRecordId(`${USER_TABLE}:${id}`);
	// console.log(recordId);
	const result = await db.select<User>(recordId);
	// console.log(result);
	return result;
};

// // TODO: better errors
// // TODO: uniqueness constraints on `email`
// export async function signup(email: string, pass: string, accountType: number): Promise<Token> {
//   const db = await initDb();
//   if (db == undefined) {
//     throw new Error("Database connection could not be initialized");
//   }

//   const access = accountType == 1 ? AccessType.Tutor : AccessType.Reviewer;
//   const vars = {
//     namespace: "grady_dev",
//     database: "dev",

//     access,
//     email,
//     pass
//   };
//   console.debug(vars);
//   try {
//     return await db.signup(vars);
//   } catch (err) {
//     console.error(err);
//     throw err;
//   }
// }

// /**
//  * Login with Access authentication
//  * @param access The kind of access used for the login
//  * @param auth The required variables for the login process of the given AccessType
//  * @returns An authentication token
//  */
// export async function login_access(access: AccessType, auth: any): Promise<Token> {
//   const db = await initDb();
//   if (db == undefined) {
//     throw new Error(ERR_MSG.db_not_init);
//   }
//   const vars = {
//     namespace: "grady_dev",
//     database: "dev",

//     access,
//     ...auth
//   };
//   try {
//     return db.signin(vars);
//   } catch (err) {
//     console.error(err);
//     throw err;
//   }
// }
