import { fail } from '@sveltejs/kit';
import { message, superValidate } from 'sveltekit-superforms/server';
import type { PageServerLoad, Actions } from './$types';
import { zod } from 'sveltekit-superforms/adapters';
import { parseAndCreateExamData } from '$lib/data/parse';
import { querySchema, uploadSchema } from './schema';

export const load: PageServerLoad = async ({ locals, cookies }) => {
	const session = cookies.get('session');

	return {
		user: locals.user,
		session,
		queryForm: await superValidate(zod(querySchema)),
		uploadForm: await superValidate(zod(uploadSchema))
	};
};

export const actions: Actions = {
	query: async ({ request }) => {
		const form = await superValidate(request, zod(querySchema));

		if (!form.valid) {
			return fail(400, { form });
		}

		try {
			// const surreal = db.getConnection();
			// const result = await surreal.query(form.data.query);
			// return message(form, { result: JSON.stringify(result, null, 2) });
		} catch (error) {
			return message(form, { error: error instanceof Error ? error.message : 'Unknown error' });
		}
	},

	upload: async ({ request }) => {
		const form = await superValidate(request, zod(uploadSchema));

		if (!form.valid) {
			return fail(400, { form });
		}

		try {
			const jsonContent = await form.data.examFile.text();
			console.log(jsonContent);
			const jsonData = JSON.parse(jsonContent);
			console.log(jsonData);
			const result = await parseAndCreateExamData(jsonData);

			return message(form, {
				type: 'success',
				text: 'Exam data successfully processed',
				data: result.moduleId.toString()
			});
		} catch (error) {
			return message(form, {
				type: 'error',
				text: error instanceof Error ? error.message : 'Failed to process exam data'
			});
		}
	}
};
